// Written by Gunanr Lindholm
// License GPLv2
#include "graf.h"
#include <string>
#include "function.h"
#include <fstream>
#include <sstream>
#include <iostream>
using namespace std;

double Graf::X(double value){
	return pictureX*(value-bbxlow)/(bbxhigh-bbxlow);
}

double Graf::Y(double value){
	return pictureY*(value-bbylow)/(bbyhigh-bbylow);
}

string Graf::v_axiscoord(double value){
	string tmp,sx,sy,svalue;
	stringstream ss1,ss2,ss3;
	double x,y;
	if (value!=0){
		x=X(0); //(pictureX*(0-bbxlow))/(bbxhigh-bbxlow);
		y=Y(value);//(pictureY*(value-bbylow))/(bbyhigh-bbylow);
		ss1<<x; sx=ss1.str();
		ss2<<y; sy=ss2.str();
		ss3<<value; svalue=ss3.str();
		tmp="\\put("+sx+","+sy+"){\\makebox(0,0)[c]{\\colorbox{white}{$"+svalue+"$}}}";
	}
	else tmp="";
	return tmp;
}

string Graf::h_axiscoord(double value){
	string tmp,sx,sy,svalue;
	stringstream ss1,ss2,ss3;
	double LOWER = 0.03; // this much lower should text be printed
	double x,y;
	if (value!=0){
		x=X(value); //(pictureX*(value-bbxlow))/(bbxhigh-bbxlow);
		y=Y(0-LOWER); //(pictureY*(0-bbylow))/(bbyhigh-bbylow);
		ss1<<x; sx=ss1.str();
		ss2<<y; sy=ss2.str();
		ss3<<value; svalue=ss3.str();
		tmp= "\\put("+sx+","+sy+"){\\makebox(0,0)[t]{$"+svalue+"$}}";//{\\colorbox{white}{$"+svalue+"$}}}";
	}
	else tmp="";
	return tmp;
}


void Graf::draw(){
	ofstream fout;
	double max,min;
	int i,j,k;
	double row,col,x,y;
	fout.open(filename.c_str());
	fout.setf(ios_base::fixed,ios_base::floatfield);
	// header
	fout<<"%% Generated by Gunnar Lindholms Funktionsritare\n";
	fout<<"%% Cartesian bounding box: ["<<bbxlow<<","<<bbylow<<"] x ["<<bbxhigh<<","<<bbyhigh<<"]\n";
	fout<<"%% Actual size: "<<unitlength*pictureX<<unitlengthunit<<" x "<<unitlength*pictureY<<unitlengthunit<<"\n";
	fout<<"%% Figure offset: right by "<<offsetX<<unitlengthunit<<", up by "<<offsetY<<unitlengthunit<<"\n\n";
	
	fout<<"\\setlength{\\unitlength}{"<<unitlength<<unitlengthunit<<"}\n";
	fout<<"\\begin{picture}("<<pictureX<<","<<pictureY<<")("<<offsetX<<","<<offsetY<<")\n";
	fout<<"\n";
	fout<<"%% Draw coordinate system\n";

	for (x=0; x>=bbxlow;x-=gridstepX)
		fout<<"\\path("<<X(x)<<","<<Y(bbylow)<<")("<<X(x)<<","<<Y(bbyhigh)<<")\n";
	for (x=0; x<=bbxhigh;x+=gridstepX)
		fout<<"\\path("<<X(x)<<","<<Y(bbylow)<<")("<<X(x)<<","<<Y(bbyhigh)<<")\n";
	for (y=0;y<=bbyhigh;y+=gridstepY)
		fout<<"\\path("<<X(bbxlow)<<","<<Y(y)<<")("<<X(bbxhigh)<<","<<Y(y)<<")\n";
	for (y=0;y>=bbylow;y-=gridstepY)
		fout<<"\\path("<<X(bbxlow)<<","<<Y(y)<<")("<<X(bbxhigh)<<","<<Y(y)<<")\n";
	fout<<"\\thicklines\n";
	fout<<"\\path("<<X(0)<<","<<Y(bbylow)<<")("<<X(0)<<","<<Y(bbyhigh)<<")\n";
	fout<<"\\path("<<X(bbxlow)<<","<<Y(0)<<")("<<X(bbxhigh)<<","<<Y(0)<<")\n";
	fout<<"\\thinlines\n";
	
	
	fout<<"\n";
	fout<<"%% Coordinate values  and axis markers\n";
	for (x=coordstepX;x<=bbxhigh;x+=coordstepX){
		//fout<<"\\put("<<x<<","<<Y(0)<<"){\\makebox(0,0)[c]{\\rule{0.5pt}{4pt}}}\n";
		fout<<h_axiscoord(x)<<"\n";
	}
	for (x=-coordstepX;x>=bbxlow;x-=coordstepX){
		//fout<<"\\put("<<x<<","<<Y(0)<<"){\\makebox(0,0)[c]{\\rule{0.5pt}{4pt}}}\n";		}
		fout<<h_axiscoord(x)<<"\n";
	}
	/*
	else
		for (x=bbxlow; x<=bbxhigh;x+=coordstepX){
			//fout<<"\\put("<<x<<","<<Y(0)<<"){\\makebox(0,0)[c]{\\rule{0.5pt}{4pt}}}\n";
			fout<<h_axiscoord(x)<<"\n";
		}*/
	for (y=coordstepY;y<=bbyhigh;y+=coordstepY){
		//fout<<"\\put("<<X(0)<<","<<Y(y)<<"){\\makebox(0,0)[c]{\\rule{4pt}{0.5pt}}}\n";
		fout<<v_axiscoord(y)<<"\n";
	}
	for (y=-coordstepY;y>=bbylow;y-=coordstepY){
		//fout<<"\\put("<<X(0)<<","<<Y(y)<<"){\\makebox(0,0)[c]{\\rule{4pt}{0.5pt}}}\n";
		fout<<v_axiscoord(y)<<"\n";
	}
	/*else
		for (y=bbylow; y<=bbyhigh;y+=coordstepY){
			//fout<<"\\put("<<X(0)<<","<<Y(y)<<"){\\makebox(0,0)[c]{\\rule{4pt}{0.5pt}}}\n";
			fout<<v_axiscoord(y)<<"\n";
	}*/
	fout<<"\n";
	
	// draws the polynomial
/*This applies if there is a discontinuity at the end	if (!leftclosed)
		poly.low+=0.001;
	if (!rightclosed)
		poly.high-=0.001;
*/
	// here
	min=poly.low;
	max=poly.high;
	if(markend){
		min=min+0.03; //the starting circle should have some space
		max=max-0.03;
	}
	x=min;
	fout<<"\\thicklines\n";	
	fout<<"\\path";
	do{
		y=poly.value(x);		
		fout<<"("<<X(x)<<","<<Y(y)<<")\n";
		x=x+stepsize;
	}while (x<=max);
	y=poly.value(max);
	fout<<"("<<X(max)<<","<<Y(y)<<")\n";
	
	// draw end of interval?
	if (markend){
		y=poly.value(poly.low);
		if (leftclosed)
			fout<<"\\put("<<X(poly.low)<<","<<Y(y)<<"){\\makebox(0,0)[c]{{\\circle*{0.12}}}}";
		else
			fout<<"\\put("<<X(poly.low)<<","<<Y(y)<<"){\\makebox(0,0)[c]{{\\circle{0.12}}}}";
		y=poly.value(poly.high);
		if (rightclosed)			
			fout<<"\\put("<<X(poly.high)<<","<<Y(y)<<"){\\makebox(0,0)[c]{{\\circle*{0.12}}}}";
		else
			fout<<"\\put("<<X(poly.high)<<","<<Y(y)<<"){\\makebox(0,0)[c]{{\\circle{0.12}}}}";
	}
		
	fout<<"\\end{picture}\n";
	fout.close();
	
}

Graf::Graf(){
	int val;int bas,faktor,delare;
	double dist;
	leftclosed=true;
	rightclosed=true;
	markend=false;
	bbxlow=-2;
	bbylow=0;
	bbxhigh=2;
	bbyhigh=4;
	offsetX=0;
	offsetY=0;

	unitlength=1;
	unitlengthunit="cm";
	coordstepX=1;
	coordstepY=1;

	
//	EPS=0.01;
//	EPSHIGH=1.0e-8;
	filename="foo.eepic";
	
	do {
		poly.find_minmax(bbyhigh,bbylow);
		bbyhigh = (int)(bbyhigh+1.0);
		bbylow = (int)(bbylow-1.0);
		if (bbyhigh<0)
			bbyhigh=0;
		if (bbylow>0)
			bbylow=0;
		
		pictureX=6;
		pictureY=6*(bbyhigh-bbylow)/(bbxhigh-bbxlow);
		// Notice, we must keep this ration below 1.8
		if (pictureY/pictureX>1.2)
			pictureY=1.2*pictureX;
		else if (pictureY/pictureX<0.5)
			pictureY=0.5*pictureX;
		
		
		dist=bbyhigh-bbylow;
		faktor=1;
		bas=1;
		while (dist/coordstepY<4){
			if (faktor==1)
				faktor=2;
			else if (faktor==2)
				faktor=4;
			else if (faktor==4){
				faktor=1;
				bas=bas*10;
			}
			delare=faktor*bas;
			coordstepY=1.0/(double)delare;
		}
		faktor=1;
		bas=1;
		
		while (dist/coordstepY>10){
			if (faktor==1)
				faktor=2;
			else if(faktor==2)
				faktor=4;
			else if (faktor==4){
				faktor=1;
				bas=bas*10;
			}
			coordstepY=(double)(faktor*bas);
		}
		
		dist=bbxhigh-bbxlow;
		faktor=1;
		bas=1;
		while (dist/coordstepX<4){
			if (faktor==1)
				faktor=2;
			else if (faktor==2)
				faktor=4;
			else if (faktor==4){
				faktor=1;
				bas=bas*10;
			}
			delare=faktor*bas;
			coordstepX=1.0/(double)delare;
		}
		faktor=1;
		bas=1;
		
		while (dist/coordstepX>10){
			if (faktor==1)
				faktor=2;
			else if(faktor==2)
				faktor=4;
			else if (faktor==4){
				faktor=1;
				bas=bas*10;
			}
			coordstepX=(double)(faktor*bas);
		}
		
		
		
		
		gridstepX=coordstepX/2;
		gridstepY=coordstepY/2;
			
		stepsize=(bbxhigh-bbxlow)/100;		
	
	
		cout<<"\n\n\n\n0. Jag r njd nu. Rita kurvan.\n\n";
		cout<<"1. ndra funktionen: ";
		poly.print();
		cout<<"\n    Intervall: ";
		if (leftclosed)
			cout<<"[ ";
		else
			cout<<"( ";
		cout<<poly.low<<" , "<<poly.high;
		if (rightclosed)
			cout<<" ]\n";
		else
			cout<<" )\n";
		cout<<"2. Ange vnster ndpunkt\n";
		cout<<"3. Ange hger ndpunkt\n\n";
		cout<<"4. Markera ";
		if (markend)
			cout<<" inte ";
		cout<<"ndpunkter\n";
		cout<<"5. Inkludera";
		if (leftclosed)
			cout<<" inte";
		cout<<" vnster ndpunkt\n";
		cout<<"6. Inkludera";
		if (rightclosed)
			cout<<" inte";
		cout<<" hger ndpunkt\n";
		cout<<"7. ndra filnamn: ("<<filename<<")\n";
		cout<<"8. ndra steg mellan X-koordinater: "<<coordstepX<<endl;
		cout<<"9. ndra steg mellan Y-koordinater: "<<coordstepY<<endl;
		cout<<"\n\nAnge val: ";
		cin>>val;
		if (val ==1)
			poly.define();
		else if(val ==2){
			cout<<"Ange den nya vnsterndpunkten: ";
			cin>>poly.low;
		}
		else if(val ==3){
			cout<<"Ange den nya hgerndpunkten: ";
			cin>>poly.high;
		}
		if (poly.low>poly.high){
			dist=poly.low;
			poly.low=poly.high;
			poly.high=dist;
		}
		bbxlow=poly.low;
		bbxhigh=poly.high;
		if (bbxhigh<0)
			bbxhigh=0;
		if (bbxlow>0)
			bbxlow=0;
			
		else if(val==4)
			markend = ! markend;
		else if (val==5)
			leftclosed = ! leftclosed;
		else if (val==6)
			rightclosed = ! rightclosed;
		else if (val==7){
			cout<<"Ange filnamnet: ";
			cin>>filename;
			if (filename.find(".eepic") == string::npos)
				filename=filename+".eepic";
		}
		else if (val==8){
			cout<<"Ange steg mellan X-koordinater: ";
			cin>>coordstepX;
		}
		else if (val==9){
			cout<<"Ange steg mellan Y-koordinater: ";
			cin>>coordstepY;
		}
	}while(val!=0);
}
